package com.arms.api.issue.schedule.service;

import com.arms.api.issue.almapi.model.entity.AlmIssueEntity;
import com.arms.api.issue.almapi.model.vo.지라이슈_데이터;
import com.arms.api.issue.almapi.model.dto.요구사항_데이터;
import com.arms.api.issue.almapi.repository.지라이슈_저장소;
import com.arms.api.issue.schedule.service.jiraissue_schedule.alm_entity_sync.req_issue.증분_요구사항_이슈_수집;
import com.arms.api.issue.schedule.service.jiraissue_schedule.alm_entity_sync.req_issue.요구사항_이슈_수집;
import com.arms.api.issue.schedule.service.jiraissue_schedule.alm_entity_sync.linked_issue.연결이슈_저장_처리;
import com.arms.api.issue.schedule.service.jiraissue_schedule.alm_entity_sync.subtask.증분_하위이슈_저장_처리;
import com.arms.api.issue.schedule.service.jiraissue_schedule.alm_entity_sync.subtask.하위이슈_저장_처리;
import com.arms.egovframework.javaservice.esframework.esquery.EsQuery;
import com.arms.egovframework.javaservice.esframework.esquery.filter.RangeQueryFilter;
import com.arms.egovframework.javaservice.esframework.esquery.filter.TermsQueryFilter;
import com.arms.egovframework.javaservice.esframework.esquery.EsQueryBuilder;
import com.arms.egovframework.javaservice.esframework.esquery.must.TermQueryMust;
import com.arms.egovframework.javaservice.esframework.factory.creator.기본_쿼리_생성기;
import com.arms.api.issue.almapi.service.이슈전략_호출;
import com.arms.egovframework.javaservice.esframework.repository.EsCommonRepositoryWrapper;

import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.elasticsearch.core.SearchHit;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.validation.Valid;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static java.util.stream.Collectors.toList;

@Slf4j
@Service("지라이슈_스케쥴_서비스")
@AllArgsConstructor
@Validated
public class 이슈_스케쥴_서비스_프로세스 extends EsCommonRepositoryWrapper<AlmIssueEntity> implements 이슈_스케쥴_서비스 {

    private final Logger 로그 = LoggerFactory.getLogger(this.getClass());

    private 지라이슈_저장소 지라이슈_저장소;

    private 이슈전략_호출 이슈전략_호출;

    @Override
    public AlmIssueEntity 이슈_조회하기(String 조회조건_아이디){
        return this.findRecentDocById(조회조건_아이디);
    }


    @Override
    public int 삭제대상_이슈_도큐먼트_삭제() {

        // 스케줄러 동작일 기준 7일 전 (working day: 5일)
        LocalDate 스케줄러_동작일 = LocalDate.now();
        LocalDate 삭제_대상 = 스케줄러_동작일.minusDays(7);
        DateTimeFormatter 날짜형식 = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        String 삭제_대상일자 = 삭제_대상.format(날짜형식);
        // 해당일에 삭제된 이슈 조회
        List<AlmIssueEntity> 삭제대상_목록 = 삭제대상이슈_조회하기(삭제_대상일자);

        if (삭제대상_목록 == null || 삭제대상_목록.isEmpty()) {
            return 0;
        }

        List<String> 삭제대상_아이디_목록 = 삭제대상_목록.stream()
                .map(AlmIssueEntity::getId)
                .collect(toList());

        // 해당 이슈 증분 데이터 포함 모든 도큐먼트 일괄 삭제
        return 지라이슈_도큐먼트_일괄삭제(삭제대상_아이디_목록);

    }

    private List<AlmIssueEntity> 삭제대상이슈_조회하기(String 삭제_대상일자) {

        EsQuery esQuery = new EsQueryBuilder()
                .bool(
                        new RangeQueryFilter("deleted.deleted_date", 삭제_대상일자, 삭제_대상일자,"fromto"),
                        new TermsQueryFilter("deleted.deleted_isDeleted","true")
                );
        return 지라이슈_저장소.findRecentHits(기본_쿼리_생성기.기본검색( esQuery).생성()).toDocs();
    }

    /*
     *   recent 필드가 true 인 es 데이터의 각 인덱스 필드를 업데이트 시킴
     * */
    @Validated
    @Override
    public int 서브테스크_상위키_필드업데이트(@Valid 요구사항_데이터 요구사항_데이터) throws Exception {

        Long 지라서버_아이디 = 요구사항_데이터.get지라서버_아이디();
        String 이슈_키 = 요구사항_데이터.get이슈_키();

        List<지라이슈_데이터> ALM_서브테스크_목록 = 이슈전략_호출.서브테스크_가져오기(지라서버_아이디, 이슈_키); // ALM에서 조회한 서브테스크 목록

        Set<String> 연결이슈_아이디_세트 = new HashSet<>();

        List<지라이슈_데이터> 연결이슈_가져오기 =이슈전략_호출.연결이슈_가져오기(요구사항_데이터);

        if (연결이슈_가져오기 != null && !연결이슈_가져오기.isEmpty()) {
            연결이슈_가져오기.stream()
                    .map(almData -> almData.조회조건_아이디(요구사항_데이터))
                    .filter(Objects::nonNull)
                    .forEach(연결이슈_아이디_세트::add);
        }

        if(!연결이슈_아이디_세트.isEmpty()) {
            AlmIssueEntity 이슈_조회하기 = 이슈_조회하기(요구사항_데이터.조회조건_아이디());

            이슈_조회하기.setLinkedIssues(new ArrayList<>(연결이슈_아이디_세트));

            this.save(이슈_조회하기);
        }


        List<SearchHit<AlmIssueEntity>> ES_서브테크_연결이슈_목록 = this.연결_및_하위_이슈목록_모든인덱스_검색하기(지라서버_아이디, 이슈_키); // ES에서 조회한 서브테스크 및 연결이슈 목록

        Map<String, 지라이슈_데이터> ALM_서브테스크_데이터_맵 = Optional.ofNullable(ALM_서브테스크_목록) // ALM 서브테스크 (키, 데이터) 맵
                .orElse(Collections.emptyList())
                .stream()
                .filter(Objects::nonNull)
                .filter(item -> item.getKey() != null)
                .collect(Collectors.toMap(지라이슈_데이터::getKey, Function.identity()));

        List<SearchHit<AlmIssueEntity>> 업데이트_데이터_목록 = new ArrayList<>();

        ES_서브테크_연결이슈_목록.forEach(서브테스크 -> {

            String ES_이슈_키 = 서브테스크.getContent().getKey();

            if (ALM_서브테스크_데이터_맵.containsKey(ES_이슈_키)) {
                // ES_서브테크_연결이슈_목록에 ALM에서 조회한 서브 테스크 가 존재한다
                // ALM의 서브 테스크 데이터에서 upperKey 조회하여 set
                String 서브테스크_상위키 = ALM_서브테스크_데이터_맵.get(ES_이슈_키).getUpperKey();

                서브테스크.getContent().setUpperKey(서브테스크_상위키);

                업데이트_데이터_목록.add(서브테스크);

            }
        });

        int 업데이트된_개수 = (int) 업데이트_데이터_목록.parallelStream()
                .filter(도큐먼트_데이터 -> {
                    try {
                        // 해당 인덱스의 도큐먼트 데이터를 업데이트 하기위함
                        this.save(도큐먼트_데이터.getContent(), 도큐먼트_데이터.getIndex());
                        return true; // 성공적으로 업데이트된 경우
                    } catch (Exception e) {
                        로그.error("필드 업데이트 중 오류 발생: " + e.getMessage());
                        return false; // 오류 발생 시 false 반환
                    }
                })
                .count(); // 성공적으로 업데이트된 개수 카운트

        return 업데이트된_개수;
    }

    @Validated
    @Override
    public List<AlmIssueEntity> 요구사항_이슈_수집(@Valid 요구사항_데이터 요구사항_데이터) throws Exception {

        요구사항_이슈_수집 요구사항_이슈_수집 =  new 요구사항_이슈_수집(요구사항_데이터);

        List<AlmIssueEntity> 지라이슈_목록 = 요구사항_이슈_수집.수집된_지라이슈_목록_가져오기();
        List<AlmIssueEntity> 저장해야할_목록 = 지라이슈_목록.stream()
                                                    .filter(a -> 삭제제외_필터링(a))
                                                    .collect(toList());
        대량이슈_추가하기(저장해야할_목록);

        지라이슈_도큐먼트_일괄삭제(
                지라이슈_목록.stream()
                        .filter(a -> 삭제대상_필터링(a))
                        .map(AlmIssueEntity::getId)
                        .collect(toList())
        );

        return 저장해야할_목록;

    }

    @Override
    public int 하위이슈_수집(요구사항_데이터 요구사항_데이터) throws Exception {

        if (요구사항_데이터.ALM_요구사항_존재_하면()) {

            List<AlmIssueEntity> 지라이슈_목록 =  new 하위이슈_저장_처리(요구사항_데이터).수집된_지라이슈_목록_가져오기();

            지라이슈_도큐먼트_일괄삭제(
                    지라이슈_목록.stream()
                            .filter(a -> 삭제대상_필터링(a))
                            .map(AlmIssueEntity::getId)
                            .collect(toList())
            );

            return 대량이슈_추가하기(
                    지라이슈_목록.stream()
                            .filter(a -> 삭제제외_필터링(a))
                            .collect(toList()));
        }
        else {
            return 0;
        }
    }

    @Override
    public int 연결이슈_수집(요구사항_데이터 요구사항_데이터) throws Exception {

        연결이슈_저장_처리 연결이슈_저장_처리 = new 연결이슈_저장_처리(요구사항_데이터);
        List<AlmIssueEntity> 저장할_지라이슈_목록 = 연결이슈_저장_처리.수집된_지라이슈_목록_가져오기();

        return 대량이슈_추가하기(저장할_지라이슈_목록);
    }

    @Validated
    @Override
    public List<AlmIssueEntity> 증분_요구사항_이슈_수집(@Valid 요구사항_데이터 요구사항_데이터) throws Exception {

        증분_요구사항_이슈_수집 증분_요구사항_이슈_저장_처리 = new 증분_요구사항_이슈_수집(요구사항_데이터);

        List<AlmIssueEntity> 지라이슈_목록 = 증분_요구사항_이슈_저장_처리.수집된_지라이슈_목록_가져오기();
        List<AlmIssueEntity> 저장해야할_목록 = 지라이슈_목록.stream()
                                                    .filter(a -> 삭제제외_필터링(a))
                                                    .collect(toList());
        대량이슈_추가하기(저장해야할_목록);

        지라이슈_도큐먼트_일괄삭제(
                지라이슈_목록.stream()
                        .filter(a -> 삭제대상_필터링(a))
                        .map(AlmIssueEntity::getId)
                        .collect(toList())
        );

        return 저장해야할_목록;

    }

    @Override
    public int 증분_하위이슈_수집(요구사항_데이터 요구사항_데이터) throws Exception {

        if(요구사항_데이터.ALM_요구사항_존재_하면()){
            List<AlmIssueEntity> 지라이슈_목록 = new 증분_하위이슈_저장_처리(요구사항_데이터).수집된_지라이슈_목록_가져오기();

            지라이슈_도큐먼트_일괄삭제(
                    지라이슈_목록.stream()
                        .filter(a -> 삭제대상_필터링(a))
                        .map(AlmIssueEntity::getId)
                        .collect(toList())
            );

            return 대량이슈_추가하기(
                        지라이슈_목록.stream()
                            .filter(a -> 삭제제외_필터링(a))
                            .collect(toList()));
        }
        else {
            return 0;
        }

    }


    private int 대량이슈_추가하기(List<AlmIssueEntity> 대량이슈_리스트) {

        List<AlmIssueEntity> 지라이슈_목록
            = 대량이슈_리스트.stream()
                .map(이슈 -> this.save(이슈))
                .collect(toList());
        return 지라이슈_목록.size();
    }

    private boolean 삭제대상_필터링(AlmIssueEntity AlmIssueEntity) {
        return AlmIssueEntity.getDeleted() != null && AlmIssueEntity.getDeleted().getIsDeleted();
    }

    private boolean 삭제제외_필터링(AlmIssueEntity AlmIssueEntity) {
        return AlmIssueEntity.getDeleted() == null || (AlmIssueEntity.getDeleted() != null && !AlmIssueEntity.getDeleted().getIsDeleted()); // null 이 아니면서, deleted false
    }

    private int 지라이슈_도큐먼트_일괄삭제(List<String> 삭제대상_아이디_목록) {

        int deletedCount = 0;

        for (String 삭제대상_아이디 : 삭제대상_아이디_목록) {

            try {
                List<SearchHit<AlmIssueEntity>> 도큐먼트_목록 = 모든인덱스에있는_이슈_조회하기(삭제대상_아이디);

                for (SearchHit<AlmIssueEntity> 도큐먼트 : 도큐먼트_목록) {
                    지라이슈_도큐먼트_삭제(도큐먼트.getId(),도큐먼트.getIndex());
                    deletedCount++;
                }
            }
            catch (Exception e) {
                로그.error("도큐먼트 일괄 삭제 중 오류 발생.", e);
            }
        }

        return deletedCount;
    }

    private String 지라이슈_도큐먼트_삭제(String 도큐먼트_아이디, String 인덱스_이름){
        return 지라이슈_저장소.deleteDocById(도큐먼트_아이디,인덱스_이름);
    }

    @Override
    public List<SearchHit<AlmIssueEntity>> 모든인덱스에있는_이슈_조회하기(String 조회조건_아이디) {

        EsQuery esQuery = new EsQueryBuilder()
                .bool(
                        new TermQueryMust("id", 조회조건_아이디)
                );

        return 지라이슈_저장소.findHitDocs(기본_쿼리_생성기.기본검색(esQuery).생성());
    }

    private List<SearchHit<AlmIssueEntity>> 연결_및_하위_이슈목록_모든인덱스_검색하기(Long 서버_아이디, String 이슈_키) {

        EsQuery esQuery = new EsQueryBuilder()
                .bool(
                        new TermsQueryFilter("parentReqKey",이슈_키)
                        ,new TermsQueryFilter("jira_server_id",서버_아이디)
                );
        return 지라이슈_저장소.findRecentHits(기본_쿼리_생성기.기본검색(esQuery).생성()).toHitDocs();
    }
}
