package com.arms.egovframework.javaservice.esframework.config;

import com.arms.config.ElasticsearchClientConfig;
import com.arms.egovframework.javaservice.esframework.repository.공통저장소;

import org.springframework.beans.factory.annotation.AnnotatedBeanDefinition;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationListener;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ClassPathScanningCandidateComponentProvider;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.event.ContextRefreshedEvent;
import org.springframework.core.type.filter.TypeFilter;
import org.springframework.data.elasticsearch.repository.config.EnableElasticsearchRepositories;

import java.lang.reflect.Field;
import java.lang.reflect.Method;

@Configuration
public class RepositoryConfiguration {

    @Autowired
    private ApplicationContext applicationContext;


    private boolean hasFieldOfType(Class<?> beanClass,Class<?> clazz) {

        Field[] fields = beanClass.getDeclaredFields();

        for (Field field : fields) {
            if (clazz.isAssignableFrom(field.getType())) {
                return true;
            }
        }
        return false;
    }

    @Bean
    public ApplicationListener<ContextRefreshedEvent> restrictAccessModifierWithRepository() {

        return event -> {
            EnableElasticsearchRepositories enableElasticsearchRepositories = ElasticsearchClientConfig.class.getAnnotation(EnableElasticsearchRepositories.class);
            String[] basePackages = enableElasticsearchRepositories.basePackages();

            ClassPathScanningCandidateComponentProvider scanner =
                new ClassPathScanningCandidateComponentProvider(false){
                    @Override
                    protected boolean isCandidateComponent(AnnotatedBeanDefinition beanDefinition) {
                        return super.isCandidateComponent(beanDefinition) || beanDefinition.getMetadata().isInterface();
                    }
                };

            TypeFilter interfaceFilter = (metadataReader, metadataReaderFactory) -> {
                try {
                    String className = metadataReader.getClassMetadata().getClassName();
                    Class<?> clazz = Class.forName(className);

                    return hasFieldOfType(clazz,공통저장소.class);
                } catch (Exception e) {
                    throw new RuntimeException("클래스 스캔 실패", e);
                }
            };

            scanner.addIncludeFilter(interfaceFilter);

            for (String basePackage : basePackages) {
                for (BeanDefinition bd : scanner.findCandidateComponents(basePackage)) {
                    try{

                        Class<?> beanClass = Class.forName(bd.getBeanClassName());

                        int modifiers = beanClass.getModifiers();

                        if(modifiers<=1){
                            throw new IllegalStateException("공통저장소를 구현한 클래스는 Public 접근제어자를 사용 하실수 없습니다." + beanClass);
                        }

                    }catch (ClassNotFoundException e){
                        throw new RuntimeException("클래스가 없습니다.",e);
                    }
                }
            }
        };
    }

    @Bean
    public ApplicationListener<ContextRefreshedEvent> restrictCommonRepository() {

        return event -> {
            EnableElasticsearchRepositories enableElasticsearchRepositories = ElasticsearchClientConfig.class.getAnnotation(EnableElasticsearchRepositories.class);
            String[] basePackages = enableElasticsearchRepositories.basePackages();

            ClassPathScanningCandidateComponentProvider scanner =
                new ClassPathScanningCandidateComponentProvider(false){
                    @Override
                    protected boolean isCandidateComponent(AnnotatedBeanDefinition beanDefinition) {
                        return super.isCandidateComponent(beanDefinition) || beanDefinition.getMetadata().isInterface();
                    }
                };

            TypeFilter interfaceFilter = (metadataReader, metadataReaderFactory) -> {
                try {
                    String className = metadataReader.getClassMetadata().getClassName();
                    Class<?> clazz = Class.forName(className);

                    return hasFieldOfType(clazz,공통저장소.class);
                } catch (Exception e) {
                    throw new RuntimeException("클래스 스캔 실패", e);
                }
            };

            scanner.addIncludeFilter(interfaceFilter);

            for (String basePackage : basePackages) {
                for (BeanDefinition bd : scanner.findCandidateComponents(basePackage)) {
                    try{

                        Class<?> beanClass = Class.forName(bd.getBeanClassName());

                        Method[] methods = beanClass.getDeclaredMethods();

                        if(methods.length>0){
                            throw new IllegalStateException("공통저장소를 구현한 클래스는 변수로 사용 하실수 없습니다." + beanClass);
                        }

                    }catch (ClassNotFoundException e){
                        throw new RuntimeException("클래스가 없습니다.",e);
                    }
                }
            }
        };
    }

    @Bean
    public ApplicationListener<ContextRefreshedEvent> validateRepositories() {

        return event -> {
            EnableElasticsearchRepositories enableElasticsearchRepositories = ElasticsearchClientConfig.class.getAnnotation(EnableElasticsearchRepositories.class);
            String[] basePackages = enableElasticsearchRepositories.basePackages();

            ClassPathScanningCandidateComponentProvider scanner =
                    new ClassPathScanningCandidateComponentProvider(false){
                        @Override
                        protected boolean isCandidateComponent(AnnotatedBeanDefinition beanDefinition) {
                            return super.isCandidateComponent(beanDefinition) || beanDefinition.getMetadata().isInterface();
                        }
                    };

            TypeFilter interfaceFilter = (metadataReader, metadataReaderFactory) -> {
                try {
                    String className = metadataReader.getClassMetadata().getClassName();
                    Class<?> clazz = Class.forName(className);
                    return 공통저장소.class.isAssignableFrom(clazz) && clazz.isInterface() && !clazz.equals(공통저장소.class);
                } catch (Exception e) {
                    throw new RuntimeException("클래스 스캔 실패", e);
                }
            };

            scanner.addIncludeFilter(interfaceFilter);

            for (String basePackage : basePackages) {

                for (BeanDefinition bd : scanner.findCandidateComponents(basePackage)) {
                    try{

                        Class<?> beanClass = Class.forName(bd.getBeanClassName());

                        Method[] methods = beanClass.getDeclaredMethods();

                        if(methods.length>0){
                            throw new IllegalStateException("공통저장소를 상속한 인터페이스는 메소드를 가질수 없습니다. " + beanClass);
                        }

                    }catch (ClassNotFoundException e){
                        throw new RuntimeException("클래스가 없습니다.",e);
                    }
                }
            }
        };
    }

}
